<?php

/**
 * BBCode with style
 *
 * @package BBCodeWithStyle
 * @version 1.5
 * @author davidhs
 * @copyright 2010-2015 by davidhs
 * @license Creative Commons Attribution 3.0 Unported License http://creativecommons.org/licenses/by/3.0/
 */

// Constants & functios for old version of PHP.
$const_array = array(
	1 => 'IMAGETYPE_GIF',
	'IMAGETYPE_JPEG',
	'IMAGETYPE_PNG',
	'IMAGETYPE_SWF',
	'IMAGETYPE_PSD',
	'IMAGETYPE_BMP',
	'IMAGETYPE_TIFF_II',
	'IMAGETYPE_TIFF_MM',
	'IMAGETYPE_JPC',
	'IMAGETYPE_JP2',
	'IMAGETYPE_JPX',
	'IMAGETYPE_JB2',
	'IMAGETYPE_SWC',
	'IMAGETYPE_IFF',
	'IMAGETYPE_WBMP',
	'IMAGETYPE_XBM',
	'IMAGETYPE_ICO',
);
foreach ($const_array as $v => $const)
{
	if (!defined($const))
		define($const, $v);
}

if (!function_exists('exif_imagetype')) // PHP > 4.3.0.
{
	function exif_imagetype($filename)
	{
		list ( , , $type) = getimagesize($filename);
		return $type;
	}
}

if (!function_exists('image_type_to_mime_type')) // PHP > 4.3.0.
{
	function image_type_to_mime_type($imagetype)
	{
		$mime_array = array(
			IMAGETYPE_GIF => 'image/gif',
			IMAGETYPE_JPEG => 'image/jpeg',
			IMAGETYPE_PNG => 'image/png',
			IMAGETYPE_SWF => 'application/x-shockwave-flash',
			IMAGETYPE_PSD => 'image/psd',
			IMAGETYPE_BMP => 'image/bmp',
			IMAGETYPE_TIFF_II => 'image/tiff',
			IMAGETYPE_TIFF_MM => 'image/tiff',
			IMAGETYPE_JPC => 'application/octet-stream',
			IMAGETYPE_JP2 => 'image/jp2',
			IMAGETYPE_JPX => 'application/octet-stream',
			IMAGETYPE_JB2 => 'application/octet-stream',
			IMAGETYPE_SWC => 'application/x-shockwave-flash',
			IMAGETYPE_IFF => 'image/iff',
			IMAGETYPE_WBMP => 'image/vnd.wap.wbmp',
			IMAGETYPE_XBM => 'image/xbm',
			IMAGETYPE_ICO => 'image/vnd.microsoft.icon',
		);
		return $mime_array[$imagetype];
	}
}

// Image type.
$suffix_array = array(
	IMAGETYPE_GIF => 'Gif',
	IMAGETYPE_JPEG => 'Jpeg',
	IMAGETYPE_PNG => 'Png',
	IMAGETYPE_SWF => '',
	IMAGETYPE_PSD => '',
	IMAGETYPE_BMP => '',
	IMAGETYPE_TIFF_II => '',
	IMAGETYPE_TIFF_MM => '',
	IMAGETYPE_JPC => '',
	IMAGETYPE_JP2 => '',
	IMAGETYPE_JPX => '',
	IMAGETYPE_JB2 => '',
	IMAGETYPE_SWC => '',
	IMAGETYPE_IFF => '',
	IMAGETYPE_WBMP => 'Wbmp',
	IMAGETYPE_XBM => 'Xbm',
	IMAGETYPE_ICO => '',
);
$type = @exif_imagetype($_GET['image']);

if ($type) // Image correct, type correct.
{
	$type2 = ($suffix_array[$type]) ? $suffix_array[$type] : 'Jpeg';

	$image_create_from_type = 'ImageCreateFrom' . $type2;
	$image_type = 'Image' . $type2;

	$img_source = $image_create_from_type($_GET['image']);
	$img_rotate = bbcwsImageRotate($img_source);
	$img_flip = bbcwsImageFlip($img_rotate);

	header('Content-Type: ' . image_type_to_mime_type($type));
	$image_type($img_flip);

	ImageDestroy($img_source);
	ImageDestroy($img_rotate);
	ImageDestroy($img_flip);
}
else // Error.
{
	$pos = strrpos($_GET['image'], '.');
	$ext = ($pos) ? strtolower(substr($_GET['image'], $pos + 1)) : '';

	header('Content-Type: ' . bbcwsExtensionToMime($ext));
	echo file_get_contents($_GET['image']);
}

// Converts an image file extension to a MIME type.
function bbcwsExtensionToMime($ext)
{
	switch ($ext)
	{
		case 'gif':
		case 'jpeg':
		case 'png':
		case 'psd':
		case 'bmp':
		case 'tiff':
		case 'jp2':
		case 'iff':
		case 'xbm':
		{
			$mime_type = 'image/' . $ext;
			break;
		}
		case 'jpg':
		{
			$mime_type = 'image/jpeg';
			break;
		}
		case 'tif':
		{
			$mime_type = 'image/tiff';
			break;
		}
		case 'wbmp':
		{
			$mime_type = 'image/vnd.wap.wbmp';
			break;
		}
		case 'ico':
		{
			$mime_type = 'image/vnd.microsoft.icon';
			break;
		}
		case 'swf':
		case 'swc':
		{
			$mime_type = 'application/x-shockwave-flash';
			break;
		}
		case 'jpc':
		case 'jpx':
		case 'jb2':
		default:
		{
			$mime_type = 'application/octet-stream';
			break;
		}
	}

	return $mime_type;
}

// Rotate image.
function bbcwsImageRotate($img_source)
{
	if (!empty($_GET['rotateangle']) && is_numeric($_GET['rotateangle']))
	{
		$color_rgb = bbcwsColorRgb();

		$color = ImageColorExact($img_source, $color_rgb[0], $color_rgb[1], $color_rgb[2]);
		if ($color == -1)
			$color = ImageColorAllocate($img_source, $color_rgb[0], $color_rgb[1], $color_rgb[2]);
		if ($color == -1 || $color === false)
			$color = ImageColorClosest($img_source, $color_rgb[0], $color_rgb[1], $color_rgb[2]);

		$img_rotate = ImageRotate($img_source, $_GET['rotateangle'], $color);
	}
	else
		$img_rotate = $img_source;

	return $img_rotate;
}

// Flip image.
function bbcwsImageFlip($img_source)
{
	$flip = '';
	if (!empty($_GET['flip']))
	{
		switch ($_GET['flip'])
		{
			case 'horizontal': case 'vertical':
			{
				$flip = $_GET['flip'];
				break;
			}
		}
	}

	if ($flip)
	{
		$width = ImageSX($img_source);
		$height = ImageSY($img_source);
		$img_flip = ImageCreateTrueColor($width, $height);

		switch ($_GET['flip'])
		{
			case 'horizontal':
			{
				for ($x = 0; $x < $width; $x++)
				{
					for ($y = 0; $y < $height; $y++)
						ImageCopy($img_flip, $img_source, $width - $x - 1, $y, $x, $y, 1, 1);
				}
				break;
			}
			case 'vertical':
			{
				for ($x = 0; $x < $width; $x++)
				{
					for ($y = 0; $y < $height; $y++)
						ImageCopy($img_flip, $img_source, $x, $height - $y - 1, $x, $y, 1, 1);
				}
				break;
			}
		}
	}
	else
		$img_flip = $img_source;

	return $img_flip;
}

// Return background color in RGB array.
function bbcwsColorRgb()
{
	$color_name_array = array(
		'aqua' => array(0, 255, 255),
		'black' => array(0, 0, 0),
		'blue' => array(0, 0, 255),
		'fuchsia' => array(255, 0, 255),
		'gray' => array(128, 128, 128),
		'green' => array(0, 128, 0),
		'lime' => array(0, 255, 0),
		'maroon' => array(128, 0, 0),
		'navy' => array(0, 0, 128),
		'olive' => array(128, 128, 0),
		'orange' => array(255, 165, 0),
		'purple' => array(128, 0, 128),
		'red' => array(255, 0, 0),
		'silver' => array(192, 192, 192),
		'teal' => array(0, 128, 128),
		'white' => array(255, 255, 255),
		'yellow' => array(255, 255, 0),
	);

	if (empty($_GET['rotatebgcolor']))
		$color_rgb = $color_name_array['black'];
	elseif ('-' == $_GET['rotatebgcolor'][0]) // #rgb or #rrggbb.
	{
		$color_hex = substr($_GET['rotatebgcolor'], 1);
		switch (strlen($color_hex))
		{
			case 0: case 1: case 2:
			{
				$color_hex = str_pad($color_hex, 3, '0');
				break;
			}
			case 4: case 5:
			{
				$color_hex = str_pad($color_hex, 6, '0');
				break;
			}
			case 3: case 6:
			{
				// VOID
				break;
			}
			default:
			{
				$color_hex = substr($color_hex, 0, 6);
				break;
			}
		}
		$color_hex = strtolower($color_hex);

		$hex_dig = '0123456789abcdef';
		$len = strlen($color_hex);
		$correct = true;
		for ($i = 0; ($i < $len) && $correct; $i++)
			$correct = !(false === strpos($hex_dig, $color_hex[$i]));

		if ($correct)
		{
			$color_hex_array = array();
			switch (strlen($color_hex))
			{
				case 3:
				{
					$color_hex_array[0] = $color_hex[0] . $color_hex[0];
					$color_hex_array[1] = $color_hex[1] . $color_hex[1];
					$color_hex_array[2] = $color_hex[2] . $color_hex[2];
					break;
				}
				case 6:
				{
					$color_hex_array[0] = substr($color_hex, 0, 2);
					$color_hex_array[1] = substr($color_hex, 2, 2);
					$color_hex_array[2] = substr($color_hex, 4, 2);
					break;
				}
			}

			$color_rgb = array();
			foreach ($color_hex_array as $hex)
				$color_rgb[] = HexDec($hex);
		}
		else
			$color_rgb = $color_name_array['black'];
	}
	elseif (is_numeric($_GET['rotatebgcolor'][0])) // r,g,b.
	{
		$color_rgb = array_pad(explode(',', $_GET['rotatebgcolor']), 3, 0);
		for ($i = 0; $i < 3; $i++)
		{
			if (!is_numeric($color_rgb[$i]) || (((int) $color_rgb[$i]) != $color_rgb[$i]) || (0 > $color_rgb[$i]) || (255 < $color_rgb[$i]))
				$color_rgb[$i] = 0;
		}
	}
	elseif (array_key_exists($_GET['rotatebgcolor'], $color_name_array)) // Name.
		$color_rgb = $color_name_array[$_GET['rotatebgcolor']];
	else // Error.
		$color_rgb = $color_name_array['black'];

	return $color_rgb;
}

?>
